@@ -29,6 +29,7 @@ from bleak import BleakScanner, BleakClient
 
 from pfxbrick import *
 from pfxbrick.pfxhelpers import *
+from pfxbrick.pfxfiles import fs_format
 
 
 def find_bricks(show_list=False):
@@ -123,6 +124,7 @@ class PFxBrick:
         self.usb_serno_str = ""
         self.dev = None
         self.is_open = False
+        self.has_bluetooth = False
         self.name = ""
 
         self.config = PFxConfig()
@@ -140,6 +142,8 @@ class PFxBrick:
         :param ser_no: optional serial number to specify a particular PFx Brick if multiple connected
         :returns: boolean indicating open session result
         """
+        if ser_no is not None:
+            ser_no = ser_no.upper()
         if not self.is_open:
             numBricks = 0
             serials = []
@@ -168,7 +172,10 @@ class PFxBrick:
                     self.usb_serno_str = self.dev.get_serial_number_string()
                     self.is_open = True
                     self.get_icd_rev()
+                    self.config.icd_rev = self.icd_rev
                     self.get_status()
+                    if self.product_id in ["A204", "A208", "A216"]:
+                        self.has_bluetooth = True
         return self.is_open
 
     def close(self):
@@ -208,28 +215,40 @@ class PFxBrick:
             self.firmware_ver = uint16_tover(res[37], res[38])
             self.firmware_build = uint16_tostr(res[39], res[40])
 
+    def status_str(self):
+        """
+        Gets top level operational status information retrieved
+        by a previous call to the get_status method into a string
+        """
+        s = []
+        s.append("USB vendor ID         : 0x%04X" % (self.usb_vid))
+        s.append("USB product ID        : 0x%04X" % (self.usb_pid))
+        s.append("USB product desc      : %s" % (self.usb_prod_str))
+        s.append("USB manufacturer      : %s" % (self.usb_manu_str))
+        s.append(
+            "PFx Brick product ID  : %s, %s" % (self.product_id, self.product_desc)
+        )
+        s.append("Serial number         : %s" % (self.serial_no))
+        s.append(
+            "Firmware version      : %s build %s"
+            % (self.firmware_ver, self.firmware_build)
+        )
+        s.append(
+            "Status                : 0x%02X %s"
+            % (self.status, get_status_str(self.status))
+        )
+        s.append(
+            "Errors                : 0x%02X %s"
+            % (self.error, get_error_str(self.error))
+        )
+        return "\n".join(s)
+
     def print_status(self):
         """
         Prints the top level operational status information retrieved
         by a previous call to the get_status method.
         """
-        print("USB vendor ID         : %04X" % (self.usb_vid))
-        print("USB product ID        : %04X" % (self.usb_pid))
-        print("USB product desc      : %s" % (self.usb_prod_str))
-        print("USB manufacturer      : %s" % (self.usb_manu_str))
-        print("PFx Brick product ID  : %s, %s" % (self.product_id, self.product_desc))
-        print("Serial number         : %s" % (self.serial_no))
-        print(
-            "Firmware version      : %s build %s"
-            % (self.firmware_ver, self.firmware_build)
-        )
-        print(
-            "Status                : %02X %s"
-            % (self.status, get_status_str(self.status))
-        )
-        print(
-            "Errors                : %02X %s" % (self.error, get_error_str(self.error))
-        )
+        print(self.status_str())
 
     def get_config(self):
         """
@@ -261,6 +280,12 @@ class PFxBrick:
         """
         res = cmd_set_config(self.dev, self.config.to_bytes())
 
+    def reset_factory_config(self):
+        """
+        Resets the PFx Brick configuration settings to factory defaults.
+        """
+        res = cmd_set_factory_defaults(self.dev)
+
     def get_name(self):
         """
         Retrieves the user defined name of the PFx Brick using
@@ -271,7 +296,8 @@ class PFxBrick:
         """
         res = cmd_get_name(self.dev)
         if res:
-            self.name = bytes(res[1:25]).decode("utf-8")
+            self.name = safe_unicode_str(res[1:25])
+        return self.name
 
     def set_name(self, name):
         """
@@ -370,6 +396,37 @@ class PFxBrick:
         """
         res = cmd_test_action(self.dev, action.to_bytes())
 
+    def find_startup_action(self, lightfx=None, soundfx=None, motorfx=None):
+        """
+        Finds a startup action with one or more specified light, motor, or
+        sound fx types.
+
+        :param lightfx: :obj:`int` optional lightfx id
+        :param soundfx: :obj:`int` optional soundfx id
+        :param motorfx: :obj:`int` optional motorfx id
+
+        :returns: :obj:`PFxAction` action data structure class matching the desired type(s)
+        """
+        req_matches = 0
+        if lightfx is not None:
+            req_matches += 1
+        if soundfx is not None:
+            req_matches += 1
+        if motorfx is not None:
+            req_matches += 1
+        for e in range(EVT_STARTUP_EVENT1, EVT_STARTUP_EVENT8 + 1):
+            matches = 0
+            action = self.get_action_by_address(e)
+            if lightfx is not None and action.lightFxId == lightfx:
+                matches += 1
+            if soundfx is not None and action.soundFxId == soundfx:
+                matches += 1
+            if motorfx is not None and action.motorActionId == motorfx:
+                matches += 1
+            if matches == req_matches:
+                return action
+        return None
+
     def set_motor_speed(self, ch, speed, duration=None):
         """
         A convenience wrapper for PFxAction().set_motor_speed
@@ -563,17 +620,57 @@ class PFxBrick:
         """
         fs_format(self.dev, quick)
 
-    def reset_factory_config(self):
+    def set_file_attributes(self, fileID, attr, mask=0x7C):
         """
-        Resets the PFx Brick configuration settings to factory defaults.
+        Sets the upper 8-bit attribute field of a file's directory entry.
+
+        Although the file attribute field is actually 16 bits, this function sets the upper
+        8 bits.  The lower 8-bits are reserved for identifying the file type, e.g. text, WAV,
+        etc.
+
+        :param fileID: :obj:`int` or :obj:`str` the file ID or filename of the file to remove
+        :param attr: :obj:`int` the attribute value to set (only the lower 16)
+        :param mask: :obj:`int` an optional bit mask applied to the attributes
         """
-        res = cmd_set_factory_defaults(self.dev)
+        fileID = self.file_id_from_str_or_int(fileID)
+        attr = attr & 0xFF
+        res = self.send_raw_icd_command(
+            [
+                PFX_CMD_FILE_DIR,
+                PFX_DIR_REQ_SET_ATTR_MASKED_ID,
+                fileID & 0xFF,
+                0x00,
+                attr,
+                0x00,
+                mask,
+            ]
+        )
+
+    def rename_file(self, fileID, new_name):
+        """
+        Renames a file on the file system.
+
+        :param fileID: :obj:`int` or :obj:`str` the file ID or filename of the file to remove
+        :param new_name: :obj:`str` new file name to apply (up to 32 characters UTF-8 encoded)
+        """
+        fileID = self.file_id_from_str_or_int(fileID)
+        mb = bytes(new_name, "utf-8")
+        name_len = min(len(mb), 32)
+        mb = mb[:name_len]
+        res = self.send_raw_icd_command(
+            [
+                PFX_CMD_FILE_DIR,
+                PFX_DIR_REQ_RENAME_FILE_ID,
+                fileID & 0xFF,
+                *mb,
+            ]
+        )
 
     def stop_script(self):
         """
         Stops all script execution.
         """
-        self.run_script(0xFF)
+        res = cmd_run_script(self.dev, 0xFF)
 
     def run_script(self, scriptfile):
         """
@@ -608,7 +705,40 @@ class PFxBrick:
         return 0xFF
 
     def get_current_state(self):
+        """
+        Returns the current state of the PFx Brick operating parameters.
+
+        :returns: :obj:`PFxState` a dataclass container with state information
+        """
         res = cmd_get_current_state(self.dev)
-        print(res)
         self.state.from_bytes(res)
         return self.state
+
+    def get_fs_state(self):
+        """
+        Returns the current state of the PFx Brick file system.
+
+        :returns: :obj:`PFxFSState` a dataclass container of file system information
+        """
+        res = self.send_raw_icd_command([PFX_CMD_FILE_GET_FS_STATE])
+        self.state.filesys.from_bytes(res)
+        return self.state.filesys
+
+    def get_bt_state(self):
+        """
+        Returns the current state of the PFx Brick Bluetooth radio module.
+
+        :returns: :obj:`PFxBTState` a dataclass container of Bluetooth module information
+        """
+        res = self.send_raw_icd_command([PFX_CMD_GET_BT_STATUS])
+        self.state.bt.from_bytes(res)
+        return self.state.bt
+
+    def send_raw_icd_command(self, msg):
+        """
+        Sends a raw ICD command message represented as a list of bytes.
+
+        :returns: :obj:`bytes` response from the PFx Brick
+        """
+        res = cmd_raw(self.dev, msg)
+        return res
