@@ -71,6 +71,18 @@ class PFxSettings:
         self.bleSessionPower = 0
         self.notchCount = 0
         self.notchBounds = [0, 0, 0, 0, 0, 0, 0]
+        self.rapidAccelThr = 0
+        self.rapidDecelThr = 0
+        self.brakeDecelThr = 0
+        self.brakeSpeedThr = 0
+
+    def __eq__(self, other):
+        for k, v in self.__dict__.items():
+            if k not in other.__dict__:
+                return False
+            if not v == other.__dict__[k]:
+                return False
+        return True
 
     def __str__(self):
         sb = []
@@ -89,8 +101,12 @@ class PFxSettings:
         sb.append("BLE Advert Power      : %s" % (self.bleAdvertPower))
         sb.append("BLE Session Power     : %s" % (self.bleSessionPower))
         sb.append("Motor sound notches   : %s" % (self.notchCount))
-        mb = "".join("{:02X} ".format(x) for x in self.notchBounds)
+        mb = "".join("0x{:02X} ".format(x) for x in self.notchBounds)
         sb.append("Motor sound bounds    : %s" % (mb))
+        sb.append("Rapid Accel Thr       : %d" % (self.rapidAccelThr))
+        sb.append("Rapid Decel Thr       : %d" % (self.rapidDecelThr))
+        sb.append("Brake Decel Thr       : %d" % (self.brakeDecelThr))
+        sb.append("Brake Speed Thr       : %d" % (self.brakeSpeedThr))
         s = "\n".join(sb)
         return s
 
@@ -129,6 +145,37 @@ class PFxMotor:
         self.vmid = 128
         self.vmax = 255
 
+    def __repr__(self):
+        s = "invert=%02X torqueComp=%02X tlgMode=%02X" % (
+            self.invert,
+            self.torqueComp,
+            self.tlgMode,
+        )
+        s = s + "vmin=%02X vmid=%02X vmax=%02X" % (self.vmin, self.vmid, self.vmax)
+        s = "%s(%s)" % ("PFxMotor." + self.__class__.__name__, s)
+        return s
+
+    def __str__(self):
+        sb = []
+        sb.append(
+            "  Invert : %s  Torque comp : %s  PF mode : %s"
+            % (self.invert, self.torqueComp, self.tlgMode)
+        )
+        sb.append("  Accel  : %s  Decel : %s" % (self.accel, self.decel))
+        sb.append(
+            "  vMin   : %s  vMid  : %s  vMax : %s" % (self.vmin, self.vmid, self.vmax)
+        )
+        s = "\n".join(sb)
+        return s
+
+    def __eq__(self, other):
+        for k, v in self.__dict__.items():
+            if k not in other.__dict__:
+                return False
+            if not v == other.__dict__[k]:
+                return False
+        return True
+
     def from_config_byte(self, byte):
         self.invert = set_with_bit(byte, PFX_CFG_MOTOR_INVERT)
         self.torqueComp = set_with_bit(byte, PFX_CFG_MOTOR_TRQCOMP)
@@ -160,29 +207,6 @@ class PFxMotor:
         v.append(self.decel)
         return v
 
-    def __repr__(self):
-        s = "invert=%02X torqueComp=%02X tlgMode=%02X" % (
-            self.invert,
-            self.torqueComp,
-            self.tlgMode,
-        )
-        s = s + "vmin=%02X vmid=%02X vmax=%02X" % (self.vmin, self.vmid, self.vmax)
-        s = "%s(%s)" % ("PFxMotor." + self.__class__.__name__, s)
-        return s
-
-    def __str__(self):
-        sb = []
-        sb.append(
-            "  Invert : %s  Torque comp : %s  PF mode : %s"
-            % (self.invert, self.torqueComp, self.tlgMode)
-        )
-        sb.append("  Accel  : %s  Decel : %s" % (self.accel, self.decel))
-        sb.append(
-            "  vMin   : %s  vMid  : %s  vMax : %s" % (self.vmin, self.vmid, self.vmax)
-        )
-        s = "\n".join(sb)
-        return s
-
 
 class PFxLights:
     """
@@ -232,17 +256,25 @@ class PFxLights:
         s = "%s(%s)" % ("PFxLights." + self.__class__.__name__, s)
         return s
 
+    def __eq__(self, other):
+        for k, v in self.__dict__.items():
+            if k not in other.__dict__:
+                return False
+            if not v == other.__dict__[k]:
+                return False
+        return True
+
     def __str__(self):
         sb = []
-        sb.append("Default brightness    : %02X" % (self.defaultBrightness))
+        sb.append("Default brightness    : 0x%02X" % (self.defaultBrightness))
         sb.append(
             "Startup brightness    : "
-            + "".join("{:02X} ".format(x) for x in self.startupBrightness)
+            + "".join("0x{:02X} ".format(x) for x in self.startupBrightness)
         )
         sb.append(
             "PF output brightness  : "
             + "".join(
-                "{:02X} ".format(x)
+                "0x{:02X} ".format(x)
                 for x in [
                     self.pfBrightnessA,
                     self.pfBrightnessB,
@@ -278,6 +310,14 @@ class PFxAudio:
         self.treble = 0
         self.defaultVolume = 0
 
+    def __eq__(self, other):
+        for k, v in self.__dict__.items():
+            if k not in other.__dict__:
+                return False
+            if not v == other.__dict__[k]:
+                return False
+        return True
+
     def __repr__(self):
         s = "drc=%d bass=%d treble=%d" % (self.audioDRC, self.bass, self.treble)
         s = "%s(%s)" % ("PFxAudio." + self.__class__.__name__, s)
@@ -289,7 +329,7 @@ class PFxAudio:
             os = "ON"
         else:
             os = "OFF"
-        s = "Audio DRC: %s  Bass: %02X  Treble: %02X" % (os, self.bass, self.treble)
+        s = "Audio DRC: %s  Bass: 0x%02X  Treble: 0x%02X" % (os, self.bass, self.treble)
         return s
 
 
@@ -311,11 +351,23 @@ class PFxConfig:
         audio (:obj:`PFxAudio`): container for audio related settings
     """
 
-    def __init__(self):
+    def __init__(self, icd_rev=None):
         self.settings = PFxSettings()
         self.motors = [PFxMotor(), PFxMotor(), PFxMotor(), PFxMotor()]
         self.lights = PFxLights()
         self.audio = PFxAudio()
+        if icd_rev is not None:
+            self.icd_rev = icd_rev
+        else:
+            self.icd_rev = "3.38"
+
+    def __eq__(self, other):
+        for k, v in self.__dict__.items():
+            if k not in other.__dict__:
+                return False
+            if not v == other.__dict__[k]:
+                return False
+        return True
 
     def from_bytes(self, msg):
         """
@@ -336,6 +388,11 @@ class PFxConfig:
         self.settings.notchBounds[4] = msg[12]
         self.settings.notchBounds[5] = msg[13]
         self.settings.notchBounds[6] = msg[14]
+        if not is_version_less_than(self.icd_rev, "3.38"):
+            self.settings.rapidAccelThr = msg[15]
+            self.settings.rapidDecelThr = msg[16]
+            self.settings.brakeDecelThr = msg[17]
+            self.settings.brakeSpeedThr = msg[18]
         self.settings.irAutoOff = msg[26]
         self.settings.bleAutoOff = msg[27]
         self.settings.bleMotorWhenDisconnect = msg[28]
@@ -377,7 +434,14 @@ class PFxConfig:
         msg.append(self.settings.notchBounds[4])
         msg.append(self.settings.notchBounds[5])
         msg.append(self.settings.notchBounds[6])
-        msg.extend([0] * 11)
+        if is_version_less_than(self.icd_rev, "3.38"):
+            msg.extend([0] * 11)
+        else:
+            msg.append(self.settings.rapidAccelThr)
+            msg.append(self.settings.rapidDecelThr)
+            msg.append(self.settings.brakeDecelThr)
+            msg.append(self.settings.brakeSpeedThr)
+            msg.extend([0] * 7)
         msg.append(self.settings.irAutoOff)
         msg.append(self.settings.bleAutoOff)
         msg.append(self.settings.bleMotorWhenDisconnect)
