@@ -23,11 +23,36 @@
 #
 # PFx Brick file system helpers
 
-import hid
 import os
 from pfxbrick import *
 from pfxbrick.pfxhelpers import *
 
+try:
+    from rich.progress import (
+        BarColumn,
+        DownloadColumn,
+        Progress,
+        TaskID,
+        TextColumn,
+        TimeRemainingColumn,
+        TransferSpeedColumn,
+    )
+
+    progress = Progress(
+        TextColumn("[white]{task.fields[filename]}", justify="right"),
+        BarColumn(bar_width=None),
+        "[progress.percentage]{task.percentage:>3.1f}%",
+        "•",
+        DownloadColumn(),
+        "•",
+        TransferSpeedColumn(),
+        "•",
+        TimeRemainingColumn(),
+    )
+    has_rich = True
+except:
+    has_rich = False
+
 
 def fs_get_fileid_from_name(hdev, name):
     """
@@ -41,7 +66,7 @@ def fs_get_fileid_from_name(hdev, name):
     p = [len(fb)]
     p.extend(fb)
     res = cmd_file_dir(hdev, PFX_DIR_REQ_GET_NAMED_FILE_ID, p)
-    if len(res) >= 3:
+    if len(res) >= 3 and not res[2] == PFX_ERR_FILE_NOT_FOUND:
         fileid = int(res[2])
     return fileid
 
@@ -117,12 +142,16 @@ def fs_copy_file_to(hdev, fid, fn, show_progress=True):
         for i in range(32 - len(nd)):
             msg.append(0)
         res = usb_transaction(hdev, msg)
-
-        if res:
-            if not fs_error_check(res[1]):
+        if not res:
+            return
+        if fs_error_check(res[1]):
+            return
+        if has_rich and show_progress:
+            with progress:
                 f = open(fn, "rb")
                 nCount = 0
                 err = False
+                transfer = progress.add_task("copy_to", filename=name, total=nBytes)
                 while (nCount < nBytes) and not err:
                     buf = f.read(61)
                     nRead = len(buf)
@@ -135,22 +164,49 @@ def fs_copy_file_to(hdev, fid, fn, show_progress=True):
                             msg.append(b)
                         res = usb_transaction(hdev, msg)
                         err = fs_error_check(res[1])
-                        if show_progress:
-                            printProgressBar(
-                                nCount,
-                                nBytes,
-                                prefix="Copying:",
-                                suffix="Complete",
-                                length=50,
-                            )
+                        progress.update(transfer, advance=nRead)
+
                 f.close()
                 msg = [PFX_CMD_FILE_CLOSE]
                 msg.append(fid)
                 res = usb_transaction(hdev, msg)
                 fs_error_check(res[1])
+            progress.remove_task(transfer)
+        else:
+            f = open(fn, "rb")
+            nCount = 0
+            err = False
+            while (nCount < nBytes) and not err:
+                buf = f.read(61)
+                nRead = len(buf)
+                nCount += nRead
+                if nRead > 0:
+                    msg = [PFX_CMD_FILE_WRITE]
+                    msg.append(fid)
+                    msg.append(nRead)
+                    for b in buf:
+                        msg.append(b)
+                    res = usb_transaction(hdev, msg)
+                    err = fs_error_check(res[1])
+                    progress.update(transfer, advance=nRead)
+                    if show_progress:
+                        printProgressBar(
+                            nCount,
+                            nBytes,
+                            prefix="Copying:",
+                            suffix="Complete",
+                            length=50,
+                        )
+            f.close()
+            msg = [PFX_CMD_FILE_CLOSE]
+            msg.append(fid)
+            res = usb_transaction(hdev, msg)
+            fs_error_check(res[1])
 
 
-def fs_copy_file_from(hdev, pfile, fn=None, show_progress=True):
+def fs_copy_file_from(
+    hdev, pfile, fn=None, show_progress=True, as_bytes=False, to_console=False
+):
     """
     File copy handler to get a file from the PFx Brick.
 
@@ -169,14 +225,19 @@ def fs_copy_file_from(hdev, pfile, fn=None, show_progress=True):
     msg.append(pfile.id)
     msg.append(0x01)  # READ mode
     res = usb_transaction(hdev, msg)
-    if res:
-        if not fs_error_check(res[1]):
+    rbytes = bytearray()
+    if not res:
+        return None
+    if fs_error_check(res[1]):
+        return None
+    if has_rich and show_progress:
+        with progress:
             nf = pfile.name
             if fn is not None:
                 nf = fn
-            f = open(nf, "wb")
             nCount = 0
             err = False
+            transfer = progress.add_task("copy_from", filename=nf, total=pfile.size)
             while (nCount < pfile.size) and not err:
                 msg = [PFX_CMD_FILE_READ]
                 msg.append(pfile.id)
@@ -189,20 +250,69 @@ def fs_copy_file_from(hdev, pfile, fn=None, show_progress=True):
                 if not err:
                     nCount += res[1]
                     b = bytes(res[2 : 2 + res[1]])
-                    f.write(b)
-                if show_progress:
-                    printProgressBar(
-                        nCount,
-                        pfile.size,
-                        prefix="Copying:",
-                        suffix="Complete",
-                        length=50,
-                    )
-            f.close()
+                    rbytes.extend(b)
+                    if to_console and not show_progress:
+                        if as_bytes:
+                            pprint_bytes(b)
+                        else:
+                            s = []
+                            for bc in b:
+                                s.append("%c" % (bc))
+                            print("".join(s), end="")
+                    progress.update(transfer, advance=res[1])
             msg = [PFX_CMD_FILE_CLOSE]
             msg.append(pfile.id)
             res = usb_transaction(hdev, msg)
             fs_error_check(res[1])
+        progress.remove_task(transfer)
+        if not as_bytes:
+            with open(nf, "wb") as f:
+                f.write(rbytes)
+        return rbytes
+    else:
+        nf = pfile.name
+        if fn is not None:
+            nf = fn
+        nCount = 0
+        err = False
+        while (nCount < pfile.size) and not err:
+            msg = [PFX_CMD_FILE_READ]
+            msg.append(pfile.id)
+            nToRead = pfile.size - nCount
+            if nToRead > 62:
+                nToRead = 62
+            msg.append(nToRead)
+            res = usb_transaction(hdev, msg)
+            err = fs_error_check(res[1])
+            if not err:
+                nCount += res[1]
+                b = bytes(res[2 : 2 + res[1]])
+                rbytes.extend(b)
+                if to_console and not show_progress:
+                    if as_bytes:
+                        pprint_bytes(b)
+                    else:
+                        s = []
+                        for bc in b:
+                            s.append("%c" % (bc))
+                        print("".join(s), end="")
+            if show_progress:
+                printProgressBar(
+                    nCount,
+                    pfile.size,
+                    prefix="Copying:",
+                    suffix="Complete",
+                    length=50,
+                )
+        msg = [PFX_CMD_FILE_CLOSE]
+        msg.append(pfile.id)
+        res = usb_transaction(hdev, msg)
+        fs_error_check(res[1])
+        if not as_bytes:
+            with open(nf, "wb") as f:
+                f.write(rbytes)
+        return rbytes
+    return None
 
 
 class PFxFile:
@@ -262,6 +372,17 @@ class PFxFile:
             return True
         return False
 
+    def has_same_crc32_as_file(self, other):
+        """
+        Checks if the CRC32 of this file is the same as a specified file on the local filesystem
+
+        :returns: True if CRC32 hash codes match
+        """
+        other_crc = get_file_crc32(other)
+        if self.crc32 == other_crc:
+            return True
+        return False
+
     def from_bytes(self, msg):
         """
         Converts the message string bytes read from the PFx Brick into
@@ -274,15 +395,19 @@ class PFxFile:
         self.userData1 = uint32_toint(msg[12:16])
         self.userData2 = uint32_toint(msg[16:20])
         self.crc32 = uint32_toint(msg[20:24])
-        sn = bytes(msg[24:56]).decode("utf-8")
-        self.name = sn.rstrip("\0")
+        self.name = safe_unicode_str(msg[24:56])
 
     def __str__(self):
         """
         Convenient human readable string of a file directory entry. This allows
         a :py:class:`PFxFile` object to be used with :obj:`str` and :obj:`print` methods.
         """
-        s = "%3d %-24s %6.1f kB %04X %08X %08X %08X" % (
+        attr_str = ""
+        if self.attributes in file_attr_dict:
+            attr_str = file_attr_dict[self.attributes]
+        elif self.id in fileid_dict:
+            attr_str = fileid_dict[self.id]
+        s = "%3d %-24s %6.1f kB %04X %08X %08X %08X %02X %s" % (
             self.id,
             self.name,
             float(self.size / 1000),
@@ -290,6 +415,8 @@ class PFxFile:
             self.userData1,
             self.userData2,
             self.crc32,
+            self.id,
+            attr_str,
         )
         return s
 
@@ -326,6 +453,7 @@ class PFxDir:
         for f in self.files:
             if f.id == fid:
                 return f
+        return None
 
     def find_available_file_id(self):
         """
@@ -342,6 +470,22 @@ class PFxDir:
                 return x
         return None
 
+    def has_file(self, fileID):
+        """
+        Determines if a specified file is on the PFx Brick file system either by
+        filename or numeric file ID.
+
+        :returns: :obj:`boolean` True or False if the file is found
+        """
+        for file in self.files:
+            if isinstance(fileID, int):
+                if file.id == fileID:
+                    return True
+            elif isinstance(fileID, str):
+                if file.name == fileID:
+                    return True
+        return False
+
     def __str__(self):
         """
         Convenient human readable string of the file directory. This allows
